mod builder;
mod entries;
mod file;

pub use builder::Builder;

use crate::archive::Handle;
use crate::ReadFilter;
use entries::Entries;
use libarchive3_sys::ffi;
use std::path::Path;

// Represents a read view of an archive
pub trait Archive: Handle + Sized {
    fn new(handle: *mut ffi::Struct_archive) -> Self;

    fn open<P: AsRef<Path>>(builder: Builder, path: P) -> crate::Result<Self>;

    fn entries(&mut self) -> Entries<'_, Self> {
        Entries::new(self)
    }

    /// Try to map a filter code to the ReadFilter enum.
    fn filter(&mut self, index: i32) -> Option<ReadFilter> {
        let res = unsafe { ffi::archive_filter_code(self.handle_mut(), index) };

        match res {
            0 => Some(ReadFilter::None),
            1 => Some(ReadFilter::Gzip),
            2 => Some(ReadFilter::Bzip2),
            3 => Some(ReadFilter::Compress),
            5 => Some(ReadFilter::Lzma),
            6 => Some(ReadFilter::Xz),
            14 => Some(ReadFilter::Zstd),
            _ => None,
        }
    }
}
